# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Type, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.images.v1 import variant_edit_params, variant_create_params
from ....types.images.v1.variant import Variant
from ....types.images.v1.variant_get_response import VariantGetResponse
from ....types.images.v1.variant_edit_response import VariantEditResponse
from ....types.images.v1.variant_create_response import VariantCreateResponse
from ....types.images.v1.variant_delete_response import VariantDeleteResponse

__all__ = ["VariantsResource", "AsyncVariantsResource"]


class VariantsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> VariantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return VariantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VariantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return VariantsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        id: str,
        options: variant_create_params.Options,
        never_require_signed_urls: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VariantCreateResponse:
        """
        Specify variants that allow you to resize images for different use cases.

        Args:
          account_id: Account identifier tag.

          options: Allows you to define image resizing sizes for different use cases.

          never_require_signed_urls: Indicates whether the variant can access an image without a signature,
              regardless of image access control.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/images/v1/variants", account_id=account_id),
            body=maybe_transform(
                {
                    "id": id,
                    "options": options,
                    "never_require_signed_urls": never_require_signed_urls,
                },
                variant_create_params.VariantCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[VariantCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[VariantCreateResponse], ResultWrapper[VariantCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Variant:
        """
        Lists existing variants.

        Args:
          account_id: Account identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/images/v1/variants", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Variant]._unwrapper,
            ),
            cast_to=cast(Type[Variant], ResultWrapper[Variant]),
        )

    def delete(
        self,
        variant_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VariantDeleteResponse:
        """
        Deleting a variant purges the cache for all images associated with the variant.

        Args:
          account_id: Account identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not variant_id:
            raise ValueError(f"Expected a non-empty value for `variant_id` but received {variant_id!r}")
        return cast(
            VariantDeleteResponse,
            self._delete(
                path_template(
                    "/accounts/{account_id}/images/v1/variants/{variant_id}",
                    account_id=account_id,
                    variant_id=variant_id,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[VariantDeleteResponse]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[VariantDeleteResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def edit(
        self,
        variant_id: str,
        *,
        account_id: str,
        options: variant_edit_params.Options,
        never_require_signed_urls: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VariantEditResponse:
        """
        Updating a variant purges the cache for all images associated with the variant.

        Args:
          account_id: Account identifier tag.

          options: Allows you to define image resizing sizes for different use cases.

          never_require_signed_urls: Indicates whether the variant can access an image without a signature,
              regardless of image access control.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not variant_id:
            raise ValueError(f"Expected a non-empty value for `variant_id` but received {variant_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/images/v1/variants/{variant_id}", account_id=account_id, variant_id=variant_id
            ),
            body=maybe_transform(
                {
                    "options": options,
                    "never_require_signed_urls": never_require_signed_urls,
                },
                variant_edit_params.VariantEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[VariantEditResponse]._unwrapper,
            ),
            cast_to=cast(Type[VariantEditResponse], ResultWrapper[VariantEditResponse]),
        )

    def get(
        self,
        variant_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VariantGetResponse:
        """
        Fetch details for a single variant.

        Args:
          account_id: Account identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not variant_id:
            raise ValueError(f"Expected a non-empty value for `variant_id` but received {variant_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/images/v1/variants/{variant_id}", account_id=account_id, variant_id=variant_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[VariantGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[VariantGetResponse], ResultWrapper[VariantGetResponse]),
        )


class AsyncVariantsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncVariantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVariantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVariantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncVariantsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        id: str,
        options: variant_create_params.Options,
        never_require_signed_urls: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VariantCreateResponse:
        """
        Specify variants that allow you to resize images for different use cases.

        Args:
          account_id: Account identifier tag.

          options: Allows you to define image resizing sizes for different use cases.

          never_require_signed_urls: Indicates whether the variant can access an image without a signature,
              regardless of image access control.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/images/v1/variants", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "id": id,
                    "options": options,
                    "never_require_signed_urls": never_require_signed_urls,
                },
                variant_create_params.VariantCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[VariantCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[VariantCreateResponse], ResultWrapper[VariantCreateResponse]),
        )

    async def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Variant:
        """
        Lists existing variants.

        Args:
          account_id: Account identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/images/v1/variants", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Variant]._unwrapper,
            ),
            cast_to=cast(Type[Variant], ResultWrapper[Variant]),
        )

    async def delete(
        self,
        variant_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VariantDeleteResponse:
        """
        Deleting a variant purges the cache for all images associated with the variant.

        Args:
          account_id: Account identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not variant_id:
            raise ValueError(f"Expected a non-empty value for `variant_id` but received {variant_id!r}")
        return cast(
            VariantDeleteResponse,
            await self._delete(
                path_template(
                    "/accounts/{account_id}/images/v1/variants/{variant_id}",
                    account_id=account_id,
                    variant_id=variant_id,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[VariantDeleteResponse]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[VariantDeleteResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def edit(
        self,
        variant_id: str,
        *,
        account_id: str,
        options: variant_edit_params.Options,
        never_require_signed_urls: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VariantEditResponse:
        """
        Updating a variant purges the cache for all images associated with the variant.

        Args:
          account_id: Account identifier tag.

          options: Allows you to define image resizing sizes for different use cases.

          never_require_signed_urls: Indicates whether the variant can access an image without a signature,
              regardless of image access control.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not variant_id:
            raise ValueError(f"Expected a non-empty value for `variant_id` but received {variant_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/images/v1/variants/{variant_id}", account_id=account_id, variant_id=variant_id
            ),
            body=await async_maybe_transform(
                {
                    "options": options,
                    "never_require_signed_urls": never_require_signed_urls,
                },
                variant_edit_params.VariantEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[VariantEditResponse]._unwrapper,
            ),
            cast_to=cast(Type[VariantEditResponse], ResultWrapper[VariantEditResponse]),
        )

    async def get(
        self,
        variant_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VariantGetResponse:
        """
        Fetch details for a single variant.

        Args:
          account_id: Account identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not variant_id:
            raise ValueError(f"Expected a non-empty value for `variant_id` but received {variant_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/images/v1/variants/{variant_id}", account_id=account_id, variant_id=variant_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[VariantGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[VariantGetResponse], ResultWrapper[VariantGetResponse]),
        )


class VariantsResourceWithRawResponse:
    def __init__(self, variants: VariantsResource) -> None:
        self._variants = variants

        self.create = to_raw_response_wrapper(
            variants.create,
        )
        self.list = to_raw_response_wrapper(
            variants.list,
        )
        self.delete = to_raw_response_wrapper(
            variants.delete,
        )
        self.edit = to_raw_response_wrapper(
            variants.edit,
        )
        self.get = to_raw_response_wrapper(
            variants.get,
        )


class AsyncVariantsResourceWithRawResponse:
    def __init__(self, variants: AsyncVariantsResource) -> None:
        self._variants = variants

        self.create = async_to_raw_response_wrapper(
            variants.create,
        )
        self.list = async_to_raw_response_wrapper(
            variants.list,
        )
        self.delete = async_to_raw_response_wrapper(
            variants.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            variants.edit,
        )
        self.get = async_to_raw_response_wrapper(
            variants.get,
        )


class VariantsResourceWithStreamingResponse:
    def __init__(self, variants: VariantsResource) -> None:
        self._variants = variants

        self.create = to_streamed_response_wrapper(
            variants.create,
        )
        self.list = to_streamed_response_wrapper(
            variants.list,
        )
        self.delete = to_streamed_response_wrapper(
            variants.delete,
        )
        self.edit = to_streamed_response_wrapper(
            variants.edit,
        )
        self.get = to_streamed_response_wrapper(
            variants.get,
        )


class AsyncVariantsResourceWithStreamingResponse:
    def __init__(self, variants: AsyncVariantsResource) -> None:
        self._variants = variants

        self.create = async_to_streamed_response_wrapper(
            variants.create,
        )
        self.list = async_to_streamed_response_wrapper(
            variants.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            variants.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            variants.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            variants.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ....._models import BaseModel

__all__ = ["CA"]


class CA(BaseModel):
    id: Optional[str] = None
    """The ID of the CA."""

    aud: Optional[str] = None
    """The Application Audience (AUD) tag.

    Identifies the application associated with the CA.
    """

    public_key: Optional[str] = None
    """The public key to add to your SSH server configuration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ...._base_client import AsyncPaginator, make_request_options
from ....types.email_security.settings import (
    allow_policy_edit_params,
    allow_policy_list_params,
    allow_policy_create_params,
)
from ....types.email_security.settings.allow_policy_get_response import AllowPolicyGetResponse
from ....types.email_security.settings.allow_policy_edit_response import AllowPolicyEditResponse
from ....types.email_security.settings.allow_policy_list_response import AllowPolicyListResponse
from ....types.email_security.settings.allow_policy_create_response import AllowPolicyCreateResponse
from ....types.email_security.settings.allow_policy_delete_response import AllowPolicyDeleteResponse

__all__ = ["AllowPoliciesResource", "AsyncAllowPoliciesResource"]


class AllowPoliciesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AllowPoliciesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AllowPoliciesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AllowPoliciesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AllowPoliciesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        is_acceptable_sender: bool,
        is_exempt_recipient: bool,
        is_regex: bool,
        is_trusted_sender: bool,
        pattern: str,
        pattern_type: Literal["EMAIL", "DOMAIN", "IP", "UNKNOWN"],
        verify_sender: bool,
        comments: Optional[str] | Omit = omit,
        is_recipient: bool | Omit = omit,
        is_sender: bool | Omit = omit,
        is_spoof: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AllowPolicyCreateResponse]:
        """Creates a new allow policy that exempts matching emails from security
        detections.

        Use with caution as this bypasses email security scanning. Policies
        can match on sender patterns and apply to specific detections or all detections.

        Args:
          account_id: Identifier.

          is_acceptable_sender: Messages from this sender will be exempted from Spam, Spoof and Bulk
              dispositions. Note - This will not exempt messages with Malicious or Suspicious
              dispositions.

          is_exempt_recipient: Messages to this recipient will bypass all detections

          is_trusted_sender: Messages from this sender will bypass all detections and link following

          pattern_type: Type of pattern matching. Note: UNKNOWN is deprecated and cannot be used when
              creating or updating policies, but may be returned for existing entries.

          verify_sender: Enforce DMARC, SPF or DKIM authentication. When on, Email Security only honors
              policies that pass authentication.

          is_recipient:
              Deprecated as of July 1, 2025. Use `is_exempt_recipient` instead. End of life:
              July 1, 2026.

          is_sender:
              Deprecated as of July 1, 2025. Use `is_trusted_sender` instead. End of life:
              July 1, 2026.

          is_spoof:
              Deprecated as of July 1, 2025. Use `is_acceptable_sender` instead. End of life:
              July 1, 2026.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/email-security/settings/allow_policies", account_id=account_id),
            body=maybe_transform(
                {
                    "is_acceptable_sender": is_acceptable_sender,
                    "is_exempt_recipient": is_exempt_recipient,
                    "is_regex": is_regex,
                    "is_trusted_sender": is_trusted_sender,
                    "pattern": pattern,
                    "pattern_type": pattern_type,
                    "verify_sender": verify_sender,
                    "comments": comments,
                    "is_recipient": is_recipient,
                    "is_sender": is_sender,
                    "is_spoof": is_spoof,
                },
                allow_policy_create_params.AllowPolicyCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AllowPolicyCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AllowPolicyCreateResponse]], ResultWrapper[AllowPolicyCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        is_acceptable_sender: bool | Omit = omit,
        is_exempt_recipient: bool | Omit = omit,
        is_trusted_sender: bool | Omit = omit,
        order: Literal["pattern", "created_at"] | Omit = omit,
        page: int | Omit = omit,
        pattern: str | Omit = omit,
        pattern_type: Literal["EMAIL", "DOMAIN", "IP", "UNKNOWN"] | Omit = omit,
        per_page: int | Omit = omit,
        search: str | Omit = omit,
        verify_sender: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[AllowPolicyListResponse]:
        """Returns a paginated list of email allow policies.

        These policies exempt matching
        emails from security detection, allowing them to bypass disposition actions.
        Supports filtering by pattern type and policy attributes.

        Args:
          account_id: Identifier.

          direction: The sorting direction.

          is_acceptable_sender: Filter to show only policies where messages from the sender are exempted from
              Spam, Spoof, and Bulk dispositions (not Malicious or Suspicious).

          is_exempt_recipient: Filter to show only policies where messages to the recipient bypass all
              detections.

          is_trusted_sender: Filter to show only policies where messages from the sender bypass all
              detections and link following.

          order: Field to sort by.

          page: Current page within paginated list of results.

          pattern_type: Type of pattern matching. Note: UNKNOWN is deprecated and cannot be used when
              creating or updating policies, but may be returned for existing entries.

          per_page: The number of results per page. Maximum value is 1000.

          search: Search term for filtering records. Behavior may change.

          verify_sender: Filter to show only policies that enforce DMARC, SPF, or DKIM authentication.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/email-security/settings/allow_policies", account_id=account_id),
            page=SyncV4PagePaginationArray[AllowPolicyListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "is_acceptable_sender": is_acceptable_sender,
                        "is_exempt_recipient": is_exempt_recipient,
                        "is_trusted_sender": is_trusted_sender,
                        "order": order,
                        "page": page,
                        "pattern": pattern,
                        "pattern_type": pattern_type,
                        "per_page": per_page,
                        "search": search,
                        "verify_sender": verify_sender,
                    },
                    allow_policy_list_params.AllowPolicyListParams,
                ),
            ),
            model=AllowPolicyListResponse,
        )

    def delete(
        self,
        policy_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AllowPolicyDeleteResponse]:
        """Removes an allow policy.

        After deletion, emails matching this pattern will be
        subject to normal security scanning and disposition actions.

        Args:
          account_id: Identifier.

          policy_id: Allow policy identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not policy_id:
            raise ValueError(f"Expected a non-empty value for `policy_id` but received {policy_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/email-security/settings/allow_policies/{policy_id}",
                account_id=account_id,
                policy_id=policy_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AllowPolicyDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AllowPolicyDeleteResponse]], ResultWrapper[AllowPolicyDeleteResponse]),
        )

    def edit(
        self,
        policy_id: str,
        *,
        account_id: str,
        comments: Optional[str] | Omit = omit,
        is_acceptable_sender: bool | Omit = omit,
        is_exempt_recipient: bool | Omit = omit,
        is_recipient: bool | Omit = omit,
        is_regex: bool | Omit = omit,
        is_sender: bool | Omit = omit,
        is_spoof: bool | Omit = omit,
        is_trusted_sender: bool | Omit = omit,
        pattern: str | Omit = omit,
        pattern_type: Literal["EMAIL", "DOMAIN", "IP", "UNKNOWN"] | Omit = omit,
        verify_sender: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AllowPolicyEditResponse]:
        """Updates an existing allow policy.

        Only provided fields will be modified. Changes
        take effect for new emails matching the pattern.

        Args:
          account_id: Identifier.

          policy_id: Allow policy identifier

          is_acceptable_sender: Messages from this sender will be exempted from Spam, Spoof and Bulk
              dispositions. Note - This will not exempt messages with Malicious or Suspicious
              dispositions.

          is_exempt_recipient: Messages to this recipient will bypass all detections

          is_recipient:
              Deprecated as of July 1, 2025. Use `is_exempt_recipient` instead. End of life:
              July 1, 2026.

          is_sender:
              Deprecated as of July 1, 2025. Use `is_trusted_sender` instead. End of life:
              July 1, 2026.

          is_spoof:
              Deprecated as of July 1, 2025. Use `is_acceptable_sender` instead. End of life:
              July 1, 2026.

          is_trusted_sender: Messages from this sender will bypass all detections and link following

          pattern_type: Type of pattern matching. Note: UNKNOWN is deprecated and cannot be used when
              creating or updating policies, but may be returned for existing entries.

          verify_sender: Enforce DMARC, SPF or DKIM authentication. When on, Email Security only honors
              policies that pass authentication.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not policy_id:
            raise ValueError(f"Expected a non-empty value for `policy_id` but received {policy_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/email-security/settings/allow_policies/{policy_id}",
                account_id=account_id,
                policy_id=policy_id,
            ),
            body=maybe_transform(
                {
                    "comments": comments,
                    "is_acceptable_sender": is_acceptable_sender,
                    "is_exempt_recipient": is_exempt_recipient,
                    "is_recipient": is_recipient,
                    "is_regex": is_regex,
                    "is_sender": is_sender,
                    "is_spoof": is_spoof,
                    "is_trusted_sender": is_trusted_sender,
                    "pattern": pattern,
                    "pattern_type": pattern_type,
                    "verify_sender": verify_sender,
                },
                allow_policy_edit_params.AllowPolicyEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AllowPolicyEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AllowPolicyEditResponse]], ResultWrapper[AllowPolicyEditResponse]),
        )

    def get(
        self,
        policy_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AllowPolicyGetResponse]:
        """
        Retrieves details for a specific allow policy including its pattern,
        dispositions that are exempted, and whether it applies to all detections.

        Args:
          account_id: Identifier.

          policy_id: Allow policy identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not policy_id:
            raise ValueError(f"Expected a non-empty value for `policy_id` but received {policy_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/email-security/settings/allow_policies/{policy_id}",
                account_id=account_id,
                policy_id=policy_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AllowPolicyGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AllowPolicyGetResponse]], ResultWrapper[AllowPolicyGetResponse]),
        )


class AsyncAllowPoliciesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAllowPoliciesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAllowPoliciesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAllowPoliciesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAllowPoliciesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        is_acceptable_sender: bool,
        is_exempt_recipient: bool,
        is_regex: bool,
        is_trusted_sender: bool,
        pattern: str,
        pattern_type: Literal["EMAIL", "DOMAIN", "IP", "UNKNOWN"],
        verify_sender: bool,
        comments: Optional[str] | Omit = omit,
        is_recipient: bool | Omit = omit,
        is_sender: bool | Omit = omit,
        is_spoof: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AllowPolicyCreateResponse]:
        """Creates a new allow policy that exempts matching emails from security
        detections.

        Use with caution as this bypasses email security scanning. Policies
        can match on sender patterns and apply to specific detections or all detections.

        Args:
          account_id: Identifier.

          is_acceptable_sender: Messages from this sender will be exempted from Spam, Spoof and Bulk
              dispositions. Note - This will not exempt messages with Malicious or Suspicious
              dispositions.

          is_exempt_recipient: Messages to this recipient will bypass all detections

          is_trusted_sender: Messages from this sender will bypass all detections and link following

          pattern_type: Type of pattern matching. Note: UNKNOWN is deprecated and cannot be used when
              creating or updating policies, but may be returned for existing entries.

          verify_sender: Enforce DMARC, SPF or DKIM authentication. When on, Email Security only honors
              policies that pass authentication.

          is_recipient:
              Deprecated as of July 1, 2025. Use `is_exempt_recipient` instead. End of life:
              July 1, 2026.

          is_sender:
              Deprecated as of July 1, 2025. Use `is_trusted_sender` instead. End of life:
              July 1, 2026.

          is_spoof:
              Deprecated as of July 1, 2025. Use `is_acceptable_sender` instead. End of life:
              July 1, 2026.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/email-security/settings/allow_policies", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "is_acceptable_sender": is_acceptable_sender,
                    "is_exempt_recipient": is_exempt_recipient,
                    "is_regex": is_regex,
                    "is_trusted_sender": is_trusted_sender,
                    "pattern": pattern,
                    "pattern_type": pattern_type,
                    "verify_sender": verify_sender,
                    "comments": comments,
                    "is_recipient": is_recipient,
                    "is_sender": is_sender,
                    "is_spoof": is_spoof,
                },
                allow_policy_create_params.AllowPolicyCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AllowPolicyCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AllowPolicyCreateResponse]], ResultWrapper[AllowPolicyCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        is_acceptable_sender: bool | Omit = omit,
        is_exempt_recipient: bool | Omit = omit,
        is_trusted_sender: bool | Omit = omit,
        order: Literal["pattern", "created_at"] | Omit = omit,
        page: int | Omit = omit,
        pattern: str | Omit = omit,
        pattern_type: Literal["EMAIL", "DOMAIN", "IP", "UNKNOWN"] | Omit = omit,
        per_page: int | Omit = omit,
        search: str | Omit = omit,
        verify_sender: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AllowPolicyListResponse, AsyncV4PagePaginationArray[AllowPolicyListResponse]]:
        """Returns a paginated list of email allow policies.

        These policies exempt matching
        emails from security detection, allowing them to bypass disposition actions.
        Supports filtering by pattern type and policy attributes.

        Args:
          account_id: Identifier.

          direction: The sorting direction.

          is_acceptable_sender: Filter to show only policies where messages from the sender are exempted from
              Spam, Spoof, and Bulk dispositions (not Malicious or Suspicious).

          is_exempt_recipient: Filter to show only policies where messages to the recipient bypass all
              detections.

          is_trusted_sender: Filter to show only policies where messages from the sender bypass all
              detections and link following.

          order: Field to sort by.

          page: Current page within paginated list of results.

          pattern_type: Type of pattern matching. Note: UNKNOWN is deprecated and cannot be used when
              creating or updating policies, but may be returned for existing entries.

          per_page: The number of results per page. Maximum value is 1000.

          search: Search term for filtering records. Behavior may change.

          verify_sender: Filter to show only policies that enforce DMARC, SPF, or DKIM authentication.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/email-security/settings/allow_policies", account_id=account_id),
            page=AsyncV4PagePaginationArray[AllowPolicyListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "is_acceptable_sender": is_acceptable_sender,
                        "is_exempt_recipient": is_exempt_recipient,
                        "is_trusted_sender": is_trusted_sender,
                        "order": order,
                        "page": page,
                        "pattern": pattern,
                        "pattern_type": pattern_type,
                        "per_page": per_page,
                        "search": search,
                        "verify_sender": verify_sender,
                    },
                    allow_policy_list_params.AllowPolicyListParams,
                ),
            ),
            model=AllowPolicyListResponse,
        )

    async def delete(
        self,
        policy_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AllowPolicyDeleteResponse]:
        """Removes an allow policy.

        After deletion, emails matching this pattern will be
        subject to normal security scanning and disposition actions.

        Args:
          account_id: Identifier.

          policy_id: Allow policy identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not policy_id:
            raise ValueError(f"Expected a non-empty value for `policy_id` but received {policy_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/email-security/settings/allow_policies/{policy_id}",
                account_id=account_id,
                policy_id=policy_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AllowPolicyDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AllowPolicyDeleteResponse]], ResultWrapper[AllowPolicyDeleteResponse]),
        )

    async def edit(
        self,
        policy_id: str,
        *,
        account_id: str,
        comments: Optional[str] | Omit = omit,
        is_acceptable_sender: bool | Omit = omit,
        is_exempt_recipient: bool | Omit = omit,
        is_recipient: bool | Omit = omit,
        is_regex: bool | Omit = omit,
        is_sender: bool | Omit = omit,
        is_spoof: bool | Omit = omit,
        is_trusted_sender: bool | Omit = omit,
        pattern: str | Omit = omit,
        pattern_type: Literal["EMAIL", "DOMAIN", "IP", "UNKNOWN"] | Omit = omit,
        verify_sender: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AllowPolicyEditResponse]:
        """Updates an existing allow policy.

        Only provided fields will be modified. Changes
        take effect for new emails matching the pattern.

        Args:
          account_id: Identifier.

          policy_id: Allow policy identifier

          is_acceptable_sender: Messages from this sender will be exempted from Spam, Spoof and Bulk
              dispositions. Note - This will not exempt messages with Malicious or Suspicious
              dispositions.

          is_exempt_recipient: Messages to this recipient will bypass all detections

          is_recipient:
              Deprecated as of July 1, 2025. Use `is_exempt_recipient` instead. End of life:
              July 1, 2026.

          is_sender:
              Deprecated as of July 1, 2025. Use `is_trusted_sender` instead. End of life:
              July 1, 2026.

          is_spoof:
              Deprecated as of July 1, 2025. Use `is_acceptable_sender` instead. End of life:
              July 1, 2026.

          is_trusted_sender: Messages from this sender will bypass all detections and link following

          pattern_type: Type of pattern matching. Note: UNKNOWN is deprecated and cannot be used when
              creating or updating policies, but may be returned for existing entries.

          verify_sender: Enforce DMARC, SPF or DKIM authentication. When on, Email Security only honors
              policies that pass authentication.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not policy_id:
            raise ValueError(f"Expected a non-empty value for `policy_id` but received {policy_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/email-security/settings/allow_policies/{policy_id}",
                account_id=account_id,
                policy_id=policy_id,
            ),
            body=await async_maybe_transform(
                {
                    "comments": comments,
                    "is_acceptable_sender": is_acceptable_sender,
                    "is_exempt_recipient": is_exempt_recipient,
                    "is_recipient": is_recipient,
                    "is_regex": is_regex,
                    "is_sender": is_sender,
                    "is_spoof": is_spoof,
                    "is_trusted_sender": is_trusted_sender,
                    "pattern": pattern,
                    "pattern_type": pattern_type,
                    "verify_sender": verify_sender,
                },
                allow_policy_edit_params.AllowPolicyEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AllowPolicyEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AllowPolicyEditResponse]], ResultWrapper[AllowPolicyEditResponse]),
        )

    async def get(
        self,
        policy_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AllowPolicyGetResponse]:
        """
        Retrieves details for a specific allow policy including its pattern,
        dispositions that are exempted, and whether it applies to all detections.

        Args:
          account_id: Identifier.

          policy_id: Allow policy identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not policy_id:
            raise ValueError(f"Expected a non-empty value for `policy_id` but received {policy_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/email-security/settings/allow_policies/{policy_id}",
                account_id=account_id,
                policy_id=policy_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AllowPolicyGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AllowPolicyGetResponse]], ResultWrapper[AllowPolicyGetResponse]),
        )


class AllowPoliciesResourceWithRawResponse:
    def __init__(self, allow_policies: AllowPoliciesResource) -> None:
        self._allow_policies = allow_policies

        self.create = to_raw_response_wrapper(
            allow_policies.create,
        )
        self.list = to_raw_response_wrapper(
            allow_policies.list,
        )
        self.delete = to_raw_response_wrapper(
            allow_policies.delete,
        )
        self.edit = to_raw_response_wrapper(
            allow_policies.edit,
        )
        self.get = to_raw_response_wrapper(
            allow_policies.get,
        )


class AsyncAllowPoliciesResourceWithRawResponse:
    def __init__(self, allow_policies: AsyncAllowPoliciesResource) -> None:
        self._allow_policies = allow_policies

        self.create = async_to_raw_response_wrapper(
            allow_policies.create,
        )
        self.list = async_to_raw_response_wrapper(
            allow_policies.list,
        )
        self.delete = async_to_raw_response_wrapper(
            allow_policies.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            allow_policies.edit,
        )
        self.get = async_to_raw_response_wrapper(
            allow_policies.get,
        )


class AllowPoliciesResourceWithStreamingResponse:
    def __init__(self, allow_policies: AllowPoliciesResource) -> None:
        self._allow_policies = allow_policies

        self.create = to_streamed_response_wrapper(
            allow_policies.create,
        )
        self.list = to_streamed_response_wrapper(
            allow_policies.list,
        )
        self.delete = to_streamed_response_wrapper(
            allow_policies.delete,
        )
        self.edit = to_streamed_response_wrapper(
            allow_policies.edit,
        )
        self.get = to_streamed_response_wrapper(
            allow_policies.get,
        )


class AsyncAllowPoliciesResourceWithStreamingResponse:
    def __init__(self, allow_policies: AsyncAllowPoliciesResource) -> None:
        self._allow_policies = allow_policies

        self.create = async_to_streamed_response_wrapper(
            allow_policies.create,
        )
        self.list = async_to_streamed_response_wrapper(
            allow_policies.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            allow_policies.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            allow_policies.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            allow_policies.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .custom_certificate import (
    CustomCertificateResource,
    AsyncCustomCertificateResource,
    CustomCertificateResourceWithRawResponse,
    AsyncCustomCertificateResourceWithRawResponse,
    CustomCertificateResourceWithStreamingResponse,
    AsyncCustomCertificateResourceWithStreamingResponse,
)
from .....types.zero_trust.gateway import (
    configuration_edit_params,
    configuration_update_params,
)
from .....types.zero_trust.gateway.configuration_get_response import ConfigurationGetResponse
from .....types.zero_trust.gateway.configuration_edit_response import ConfigurationEditResponse
from .....types.zero_trust.gateway.configuration_update_response import ConfigurationUpdateResponse
from .....types.zero_trust.gateway.gateway_configuration_settings_param import GatewayConfigurationSettingsParam

__all__ = ["ConfigurationsResource", "AsyncConfigurationsResource"]


class ConfigurationsResource(SyncAPIResource):
    @cached_property
    def custom_certificate(self) -> CustomCertificateResource:
        return CustomCertificateResource(self._client)

    @cached_property
    def with_raw_response(self) -> ConfigurationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ConfigurationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ConfigurationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ConfigurationsResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        account_id: str,
        settings: GatewayConfigurationSettingsParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ConfigurationUpdateResponse]:
        """
        Update the current Zero Trust account configuration.

        Args:
          settings: Specify account settings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/gateway/configuration", account_id=account_id),
            body=maybe_transform({"settings": settings}, configuration_update_params.ConfigurationUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ConfigurationUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ConfigurationUpdateResponse]], ResultWrapper[ConfigurationUpdateResponse]),
        )

    def edit(
        self,
        *,
        account_id: str,
        settings: GatewayConfigurationSettingsParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ConfigurationEditResponse]:
        """
        Update (PATCH) a single subcollection of settings such as `antivirus`,
        `tls_decrypt`, `activity_log`, `block_page`, `browser_isolation`, `fips`,
        `body_scanning`, or `certificate` without updating the entire configuration
        object. This endpoint returns an error if any settings collection lacks proper
        configuration.

        Args:
          settings: Specify account settings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._patch(
            path_template("/accounts/{account_id}/gateway/configuration", account_id=account_id),
            body=maybe_transform({"settings": settings}, configuration_edit_params.ConfigurationEditParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ConfigurationEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ConfigurationEditResponse]], ResultWrapper[ConfigurationEditResponse]),
        )

    def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ConfigurationGetResponse]:
        """
        Retrieve the current Zero Trust account configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/gateway/configuration", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ConfigurationGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ConfigurationGetResponse]], ResultWrapper[ConfigurationGetResponse]),
        )


class AsyncConfigurationsResource(AsyncAPIResource):
    @cached_property
    def custom_certificate(self) -> AsyncCustomCertificateResource:
        return AsyncCustomCertificateResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncConfigurationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncConfigurationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncConfigurationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncConfigurationsResourceWithStreamingResponse(self)

    async def update(
        self,
        *,
        account_id: str,
        settings: GatewayConfigurationSettingsParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ConfigurationUpdateResponse]:
        """
        Update the current Zero Trust account configuration.

        Args:
          settings: Specify account settings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/gateway/configuration", account_id=account_id),
            body=await async_maybe_transform(
                {"settings": settings}, configuration_update_params.ConfigurationUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ConfigurationUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ConfigurationUpdateResponse]], ResultWrapper[ConfigurationUpdateResponse]),
        )

    async def edit(
        self,
        *,
        account_id: str,
        settings: GatewayConfigurationSettingsParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ConfigurationEditResponse]:
        """
        Update (PATCH) a single subcollection of settings such as `antivirus`,
        `tls_decrypt`, `activity_log`, `block_page`, `browser_isolation`, `fips`,
        `body_scanning`, or `certificate` without updating the entire configuration
        object. This endpoint returns an error if any settings collection lacks proper
        configuration.

        Args:
          settings: Specify account settings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._patch(
            path_template("/accounts/{account_id}/gateway/configuration", account_id=account_id),
            body=await async_maybe_transform({"settings": settings}, configuration_edit_params.ConfigurationEditParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ConfigurationEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ConfigurationEditResponse]], ResultWrapper[ConfigurationEditResponse]),
        )

    async def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ConfigurationGetResponse]:
        """
        Retrieve the current Zero Trust account configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/gateway/configuration", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ConfigurationGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ConfigurationGetResponse]], ResultWrapper[ConfigurationGetResponse]),
        )


class ConfigurationsResourceWithRawResponse:
    def __init__(self, configurations: ConfigurationsResource) -> None:
        self._configurations = configurations

        self.update = to_raw_response_wrapper(
            configurations.update,
        )
        self.edit = to_raw_response_wrapper(
            configurations.edit,
        )
        self.get = to_raw_response_wrapper(
            configurations.get,
        )

    @cached_property
    def custom_certificate(self) -> CustomCertificateResourceWithRawResponse:
        return CustomCertificateResourceWithRawResponse(self._configurations.custom_certificate)


class AsyncConfigurationsResourceWithRawResponse:
    def __init__(self, configurations: AsyncConfigurationsResource) -> None:
        self._configurations = configurations

        self.update = async_to_raw_response_wrapper(
            configurations.update,
        )
        self.edit = async_to_raw_response_wrapper(
            configurations.edit,
        )
        self.get = async_to_raw_response_wrapper(
            configurations.get,
        )

    @cached_property
    def custom_certificate(self) -> AsyncCustomCertificateResourceWithRawResponse:
        return AsyncCustomCertificateResourceWithRawResponse(self._configurations.custom_certificate)


class ConfigurationsResourceWithStreamingResponse:
    def __init__(self, configurations: ConfigurationsResource) -> None:
        self._configurations = configurations

        self.update = to_streamed_response_wrapper(
            configurations.update,
        )
        self.edit = to_streamed_response_wrapper(
            configurations.edit,
        )
        self.get = to_streamed_response_wrapper(
            configurations.get,
        )

    @cached_property
    def custom_certificate(self) -> CustomCertificateResourceWithStreamingResponse:
        return CustomCertificateResourceWithStreamingResponse(self._configurations.custom_certificate)


class AsyncConfigurationsResourceWithStreamingResponse:
    def __init__(self, configurations: AsyncConfigurationsResource) -> None:
        self._configurations = configurations

        self.update = async_to_streamed_response_wrapper(
            configurations.update,
        )
        self.edit = async_to_streamed_response_wrapper(
            configurations.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            configurations.get,
        )

    @cached_property
    def custom_certificate(self) -> AsyncCustomCertificateResourceWithStreamingResponse:
        return AsyncCustomCertificateResourceWithStreamingResponse(self._configurations.custom_certificate)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.ai_security import custom_topic_update_params
from ...types.ai_security.custom_topic_get_response import CustomTopicGetResponse
from ...types.ai_security.custom_topic_update_response import CustomTopicUpdateResponse

__all__ = ["CustomTopicsResource", "AsyncCustomTopicsResource"]


class CustomTopicsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CustomTopicsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return CustomTopicsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CustomTopicsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return CustomTopicsResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        zone_id: str,
        topics: Iterable[custom_topic_update_params.Topic] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomTopicUpdateResponse:
        """
        Set the AI Security for Apps custom topic categories for a zone.

        A maximum of 20 custom topics can be configured per zone. Each topic label must
        be 2–20 characters using only lowercase letters (a–z), digits (0–9), and
        hyphens. Each topic description must be 2–50 printable ASCII characters.

        Changes can take up to a minute to propagate to the zone.

        Args:
          zone_id: Defines the zone.

          topics: Custom topic categories for AI Security for Apps content detection.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._put(
            path_template("/zones/{zone_id}/ai-security/custom-topics", zone_id=zone_id),
            body=maybe_transform({"topics": topics}, custom_topic_update_params.CustomTopicUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CustomTopicUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[CustomTopicUpdateResponse], ResultWrapper[CustomTopicUpdateResponse]),
        )

    def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomTopicGetResponse:
        """
        Get the AI Security for Apps custom topic categories for a zone.

        Args:
          zone_id: Defines the zone.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/ai-security/custom-topics", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CustomTopicGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[CustomTopicGetResponse], ResultWrapper[CustomTopicGetResponse]),
        )


class AsyncCustomTopicsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCustomTopicsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCustomTopicsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCustomTopicsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncCustomTopicsResourceWithStreamingResponse(self)

    async def update(
        self,
        *,
        zone_id: str,
        topics: Iterable[custom_topic_update_params.Topic] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomTopicUpdateResponse:
        """
        Set the AI Security for Apps custom topic categories for a zone.

        A maximum of 20 custom topics can be configured per zone. Each topic label must
        be 2–20 characters using only lowercase letters (a–z), digits (0–9), and
        hyphens. Each topic description must be 2–50 printable ASCII characters.

        Changes can take up to a minute to propagate to the zone.

        Args:
          zone_id: Defines the zone.

          topics: Custom topic categories for AI Security for Apps content detection.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._put(
            path_template("/zones/{zone_id}/ai-security/custom-topics", zone_id=zone_id),
            body=await async_maybe_transform({"topics": topics}, custom_topic_update_params.CustomTopicUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CustomTopicUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[CustomTopicUpdateResponse], ResultWrapper[CustomTopicUpdateResponse]),
        )

    async def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomTopicGetResponse:
        """
        Get the AI Security for Apps custom topic categories for a zone.

        Args:
          zone_id: Defines the zone.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/ai-security/custom-topics", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CustomTopicGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[CustomTopicGetResponse], ResultWrapper[CustomTopicGetResponse]),
        )


class CustomTopicsResourceWithRawResponse:
    def __init__(self, custom_topics: CustomTopicsResource) -> None:
        self._custom_topics = custom_topics

        self.update = to_raw_response_wrapper(
            custom_topics.update,
        )
        self.get = to_raw_response_wrapper(
            custom_topics.get,
        )


class AsyncCustomTopicsResourceWithRawResponse:
    def __init__(self, custom_topics: AsyncCustomTopicsResource) -> None:
        self._custom_topics = custom_topics

        self.update = async_to_raw_response_wrapper(
            custom_topics.update,
        )
        self.get = async_to_raw_response_wrapper(
            custom_topics.get,
        )


class CustomTopicsResourceWithStreamingResponse:
    def __init__(self, custom_topics: CustomTopicsResource) -> None:
        self._custom_topics = custom_topics

        self.update = to_streamed_response_wrapper(
            custom_topics.update,
        )
        self.get = to_streamed_response_wrapper(
            custom_topics.get,
        )


class AsyncCustomTopicsResourceWithStreamingResponse:
    def __init__(self, custom_topics: AsyncCustomTopicsResource) -> None:
        self._custom_topics = custom_topics

        self.update = async_to_streamed_response_wrapper(
            custom_topics.update,
        )
        self.get = async_to_streamed_response_wrapper(
            custom_topics.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Union, Optional, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from .raw import (
    RawResource,
    AsyncRawResource,
    RawResourceWithRawResponse,
    AsyncRawResourceWithRawResponse,
    RawResourceWithStreamingResponse,
    AsyncRawResourceWithStreamingResponse,
)
from .move import (
    MoveResource,
    AsyncMoveResource,
    MoveResourceWithRawResponse,
    AsyncMoveResourceWithRawResponse,
    MoveResourceWithStreamingResponse,
    AsyncMoveResourceWithStreamingResponse,
)
from .trace import (
    TraceResource,
    AsyncTraceResource,
    TraceResourceWithRawResponse,
    AsyncTraceResourceWithRawResponse,
    TraceResourceWithStreamingResponse,
    AsyncTraceResourceWithStreamingResponse,
)
from .preview import (
    PreviewResource,
    AsyncPreviewResource,
    PreviewResourceWithRawResponse,
    AsyncPreviewResourceWithRawResponse,
    PreviewResourceWithStreamingResponse,
    AsyncPreviewResourceWithStreamingResponse,
)
from .release import (
    ReleaseResource,
    AsyncReleaseResource,
    ReleaseResourceWithRawResponse,
    AsyncReleaseResourceWithRawResponse,
    ReleaseResourceWithStreamingResponse,
    AsyncReleaseResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from .detections import (
    DetectionsResource,
    AsyncDetectionsResource,
    DetectionsResourceWithRawResponse,
    AsyncDetectionsResourceWithRawResponse,
    DetectionsResourceWithStreamingResponse,
    AsyncDetectionsResourceWithStreamingResponse,
)
from .reclassify import (
    ReclassifyResource,
    AsyncReclassifyResource,
    ReclassifyResourceWithRawResponse,
    AsyncReclassifyResourceWithRawResponse,
    ReclassifyResourceWithStreamingResponse,
    AsyncReclassifyResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ...._base_client import AsyncPaginator, make_request_options
from ....types.email_security import investigate_get_params, investigate_list_params
from ....types.email_security.investigate_get_response import InvestigateGetResponse
from ....types.email_security.investigate_list_response import InvestigateListResponse

__all__ = ["InvestigateResource", "AsyncInvestigateResource"]


class InvestigateResource(SyncAPIResource):
    @cached_property
    def detections(self) -> DetectionsResource:
        return DetectionsResource(self._client)

    @cached_property
    def preview(self) -> PreviewResource:
        return PreviewResource(self._client)

    @cached_property
    def raw(self) -> RawResource:
        return RawResource(self._client)

    @cached_property
    def trace(self) -> TraceResource:
        return TraceResource(self._client)

    @cached_property
    def move(self) -> MoveResource:
        return MoveResource(self._client)

    @cached_property
    def reclassify(self) -> ReclassifyResource:
        return ReclassifyResource(self._client)

    @cached_property
    def release(self) -> ReleaseResource:
        return ReleaseResource(self._client)

    @cached_property
    def with_raw_response(self) -> InvestigateResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return InvestigateResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InvestigateResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return InvestigateResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        action_log: bool | Omit = omit,
        alert_id: str | Omit = omit,
        cursor: str | Omit = omit,
        delivery_status: Literal["delivered", "moved", "quarantined", "rejected", "deferred", "bounced", "queued"]
        | Omit = omit,
        detections_only: bool | Omit = omit,
        domain: str | Omit = omit,
        end: Union[str, datetime] | Omit = omit,
        final_disposition: Literal["MALICIOUS", "SUSPICIOUS", "SPOOF", "SPAM", "BULK", "NONE"] | Omit = omit,
        message_action: Literal["PREVIEW", "QUARANTINE_RELEASED", "MOVED"] | Omit = omit,
        message_id: str | Omit = omit,
        metric: str | Omit = omit,
        page: Optional[int] | Omit = omit,
        per_page: int | Omit = omit,
        query: str | Omit = omit,
        recipient: str | Omit = omit,
        sender: str | Omit = omit,
        start: Union[str, datetime] | Omit = omit,
        subject: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[InvestigateListResponse]:
        """
        Returns information for each email that matches the search parameter(s).

        Args:
          account_id: Identifier.

          action_log: Whether to include the message action log in the response.

          delivery_status: Delivery status to filter by.

          detections_only: Whether to include only detections in search results.

          domain: Sender domains to filter by.

          end: The end of the search date range. Defaults to `now`.

          final_disposition: Dispositions to filter by.

          message_action: Message actions to filter by.

          page: Deprecated: Use cursor pagination instead. End of life: November 1, 2026.

          per_page: The number of results per page. Maximum value is 1000.

          query: Space-delimited search term. Case-insensitive.

          start: The beginning of the search date range. Defaults to `now - 30 days`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/email-security/investigate", account_id=account_id),
            page=SyncV4PagePaginationArray[InvestigateListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "action_log": action_log,
                        "alert_id": alert_id,
                        "cursor": cursor,
                        "delivery_status": delivery_status,
                        "detections_only": detections_only,
                        "domain": domain,
                        "end": end,
                        "final_disposition": final_disposition,
                        "message_action": message_action,
                        "message_id": message_id,
                        "metric": metric,
                        "page": page,
                        "per_page": per_page,
                        "query": query,
                        "recipient": recipient,
                        "sender": sender,
                        "start": start,
                        "subject": subject,
                    },
                    investigate_list_params.InvestigateListParams,
                ),
            ),
            model=InvestigateListResponse,
        )

    def get(
        self,
        investigate_id: str,
        *,
        account_id: str,
        submission: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InvestigateGetResponse:
        """
        Retrieves comprehensive details for a specific email message including headers,
        recipients, sender information, and current quarantine status. Use the
        investigate_id from search results to fetch detailed information.

        Args:
          account_id: Identifier.

          investigate_id: Unique identifier for a message retrieved from investigation

          submission: When true, search the submissions datastore only. When false or omitted, search
              the regular datastore only.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not investigate_id:
            raise ValueError(f"Expected a non-empty value for `investigate_id` but received {investigate_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/email-security/investigate/{investigate_id}",
                account_id=account_id,
                investigate_id=investigate_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"submission": submission}, investigate_get_params.InvestigateGetParams),
                post_parser=ResultWrapper[InvestigateGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[InvestigateGetResponse], ResultWrapper[InvestigateGetResponse]),
        )


class AsyncInvestigateResource(AsyncAPIResource):
    @cached_property
    def detections(self) -> AsyncDetectionsResource:
        return AsyncDetectionsResource(self._client)

    @cached_property
    def preview(self) -> AsyncPreviewResource:
        return AsyncPreviewResource(self._client)

    @cached_property
    def raw(self) -> AsyncRawResource:
        return AsyncRawResource(self._client)

    @cached_property
    def trace(self) -> AsyncTraceResource:
        return AsyncTraceResource(self._client)

    @cached_property
    def move(self) -> AsyncMoveResource:
        return AsyncMoveResource(self._client)

    @cached_property
    def reclassify(self) -> AsyncReclassifyResource:
        return AsyncReclassifyResource(self._client)

    @cached_property
    def release(self) -> AsyncReleaseResource:
        return AsyncReleaseResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncInvestigateResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInvestigateResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInvestigateResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncInvestigateResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        action_log: bool | Omit = omit,
        alert_id: str | Omit = omit,
        cursor: str | Omit = omit,
        delivery_status: Literal["delivered", "moved", "quarantined", "rejected", "deferred", "bounced", "queued"]
        | Omit = omit,
        detections_only: bool | Omit = omit,
        domain: str | Omit = omit,
        end: Union[str, datetime] | Omit = omit,
        final_disposition: Literal["MALICIOUS", "SUSPICIOUS", "SPOOF", "SPAM", "BULK", "NONE"] | Omit = omit,
        message_action: Literal["PREVIEW", "QUARANTINE_RELEASED", "MOVED"] | Omit = omit,
        message_id: str | Omit = omit,
        metric: str | Omit = omit,
        page: Optional[int] | Omit = omit,
        per_page: int | Omit = omit,
        query: str | Omit = omit,
        recipient: str | Omit = omit,
        sender: str | Omit = omit,
        start: Union[str, datetime] | Omit = omit,
        subject: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[InvestigateListResponse, AsyncV4PagePaginationArray[InvestigateListResponse]]:
        """
        Returns information for each email that matches the search parameter(s).

        Args:
          account_id: Identifier.

          action_log: Whether to include the message action log in the response.

          delivery_status: Delivery status to filter by.

          detections_only: Whether to include only detections in search results.

          domain: Sender domains to filter by.

          end: The end of the search date range. Defaults to `now`.

          final_disposition: Dispositions to filter by.

          message_action: Message actions to filter by.

          page: Deprecated: Use cursor pagination instead. End of life: November 1, 2026.

          per_page: The number of results per page. Maximum value is 1000.

          query: Space-delimited search term. Case-insensitive.

          start: The beginning of the search date range. Defaults to `now - 30 days`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/email-security/investigate", account_id=account_id),
            page=AsyncV4PagePaginationArray[InvestigateListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "action_log": action_log,
                        "alert_id": alert_id,
                        "cursor": cursor,
                        "delivery_status": delivery_status,
                        "detections_only": detections_only,
                        "domain": domain,
                        "end": end,
                        "final_disposition": final_disposition,
                        "message_action": message_action,
                        "message_id": message_id,
                        "metric": metric,
                        "page": page,
                        "per_page": per_page,
                        "query": query,
                        "recipient": recipient,
                        "sender": sender,
                        "start": start,
                        "subject": subject,
                    },
                    investigate_list_params.InvestigateListParams,
                ),
            ),
            model=InvestigateListResponse,
        )

    async def get(
        self,
        investigate_id: str,
        *,
        account_id: str,
        submission: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InvestigateGetResponse:
        """
        Retrieves comprehensive details for a specific email message including headers,
        recipients, sender information, and current quarantine status. Use the
        investigate_id from search results to fetch detailed information.

        Args:
          account_id: Identifier.

          investigate_id: Unique identifier for a message retrieved from investigation

          submission: When true, search the submissions datastore only. When false or omitted, search
              the regular datastore only.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not investigate_id:
            raise ValueError(f"Expected a non-empty value for `investigate_id` but received {investigate_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/email-security/investigate/{investigate_id}",
                account_id=account_id,
                investigate_id=investigate_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"submission": submission}, investigate_get_params.InvestigateGetParams
                ),
                post_parser=ResultWrapper[InvestigateGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[InvestigateGetResponse], ResultWrapper[InvestigateGetResponse]),
        )


class InvestigateResourceWithRawResponse:
    def __init__(self, investigate: InvestigateResource) -> None:
        self._investigate = investigate

        self.list = to_raw_response_wrapper(
            investigate.list,
        )
        self.get = to_raw_response_wrapper(
            investigate.get,
        )

    @cached_property
    def detections(self) -> DetectionsResourceWithRawResponse:
        return DetectionsResourceWithRawResponse(self._investigate.detections)

    @cached_property
    def preview(self) -> PreviewResourceWithRawResponse:
        return PreviewResourceWithRawResponse(self._investigate.preview)

    @cached_property
    def raw(self) -> RawResourceWithRawResponse:
        return RawResourceWithRawResponse(self._investigate.raw)

    @cached_property
    def trace(self) -> TraceResourceWithRawResponse:
        return TraceResourceWithRawResponse(self._investigate.trace)

    @cached_property
    def move(self) -> MoveResourceWithRawResponse:
        return MoveResourceWithRawResponse(self._investigate.move)

    @cached_property
    def reclassify(self) -> ReclassifyResourceWithRawResponse:
        return ReclassifyResourceWithRawResponse(self._investigate.reclassify)

    @cached_property
    def release(self) -> ReleaseResourceWithRawResponse:
        return ReleaseResourceWithRawResponse(self._investigate.release)


class AsyncInvestigateResourceWithRawResponse:
    def __init__(self, investigate: AsyncInvestigateResource) -> None:
        self._investigate = investigate

        self.list = async_to_raw_response_wrapper(
            investigate.list,
        )
        self.get = async_to_raw_response_wrapper(
            investigate.get,
        )

    @cached_property
    def detections(self) -> AsyncDetectionsResourceWithRawResponse:
        return AsyncDetectionsResourceWithRawResponse(self._investigate.detections)

    @cached_property
    def preview(self) -> AsyncPreviewResourceWithRawResponse:
        return AsyncPreviewResourceWithRawResponse(self._investigate.preview)

    @cached_property
    def raw(self) -> AsyncRawResourceWithRawResponse:
        return AsyncRawResourceWithRawResponse(self._investigate.raw)

    @cached_property
    def trace(self) -> AsyncTraceResourceWithRawResponse:
        return AsyncTraceResourceWithRawResponse(self._investigate.trace)

    @cached_property
    def move(self) -> AsyncMoveResourceWithRawResponse:
        return AsyncMoveResourceWithRawResponse(self._investigate.move)

    @cached_property
    def reclassify(self) -> AsyncReclassifyResourceWithRawResponse:
        return AsyncReclassifyResourceWithRawResponse(self._investigate.reclassify)

    @cached_property
    def release(self) -> AsyncReleaseResourceWithRawResponse:
        return AsyncReleaseResourceWithRawResponse(self._investigate.release)


class InvestigateResourceWithStreamingResponse:
    def __init__(self, investigate: InvestigateResource) -> None:
        self._investigate = investigate

        self.list = to_streamed_response_wrapper(
            investigate.list,
        )
        self.get = to_streamed_response_wrapper(
            investigate.get,
        )

    @cached_property
    def detections(self) -> DetectionsResourceWithStreamingResponse:
        return DetectionsResourceWithStreamingResponse(self._investigate.detections)

    @cached_property
    def preview(self) -> PreviewResourceWithStreamingResponse:
        return PreviewResourceWithStreamingResponse(self._investigate.preview)

    @cached_property
    def raw(self) -> RawResourceWithStreamingResponse:
        return RawResourceWithStreamingResponse(self._investigate.raw)

    @cached_property
    def trace(self) -> TraceResourceWithStreamingResponse:
        return TraceResourceWithStreamingResponse(self._investigate.trace)

    @cached_property
    def move(self) -> MoveResourceWithStreamingResponse:
        return MoveResourceWithStreamingResponse(self._investigate.move)

    @cached_property
    def reclassify(self) -> ReclassifyResourceWithStreamingResponse:
        return ReclassifyResourceWithStreamingResponse(self._investigate.reclassify)

    @cached_property
    def release(self) -> ReleaseResourceWithStreamingResponse:
        return ReleaseResourceWithStreamingResponse(self._investigate.release)


class AsyncInvestigateResourceWithStreamingResponse:
    def __init__(self, investigate: AsyncInvestigateResource) -> None:
        self._investigate = investigate

        self.list = async_to_streamed_response_wrapper(
            investigate.list,
        )
        self.get = async_to_streamed_response_wrapper(
            investigate.get,
        )

    @cached_property
    def detections(self) -> AsyncDetectionsResourceWithStreamingResponse:
        return AsyncDetectionsResourceWithStreamingResponse(self._investigate.detections)

    @cached_property
    def preview(self) -> AsyncPreviewResourceWithStreamingResponse:
        return AsyncPreviewResourceWithStreamingResponse(self._investigate.preview)

    @cached_property
    def raw(self) -> AsyncRawResourceWithStreamingResponse:
        return AsyncRawResourceWithStreamingResponse(self._investigate.raw)

    @cached_property
    def trace(self) -> AsyncTraceResourceWithStreamingResponse:
        return AsyncTraceResourceWithStreamingResponse(self._investigate.trace)

    @cached_property
    def move(self) -> AsyncMoveResourceWithStreamingResponse:
        return AsyncMoveResourceWithStreamingResponse(self._investigate.move)

    @cached_property
    def reclassify(self) -> AsyncReclassifyResourceWithStreamingResponse:
        return AsyncReclassifyResourceWithStreamingResponse(self._investigate.reclassify)

    @cached_property
    def release(self) -> AsyncReleaseResourceWithStreamingResponse:
        return AsyncReleaseResourceWithStreamingResponse(self._investigate.release)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["PageShieldUpdateResponse"]


class PageShieldUpdateResponse(BaseModel):
    enabled: bool
    """When true, indicates that Page Shield is enabled."""

    updated_at: str
    """The timestamp of when Page Shield was last updated."""

    use_cloudflare_reporting_endpoint: bool
    """
    When true, CSP reports will be sent to
    https://csp-reporting.cloudflare.com/cdn-cgi/script_monitor/report
    """

    use_connection_url_path: bool
    """When true, the paths associated with connections URLs will also be analyzed."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from datetime import date
from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = [
    "AuditListParams",
    "ID",
    "ActionResult",
    "ActionType",
    "ActorContext",
    "ActorEmail",
    "ActorID",
    "ActorIPAddress",
    "ActorTokenID",
    "ActorTokenName",
    "ActorType",
    "RawCfRayID",
    "RawMethod",
    "RawStatusCode",
    "RawURI",
    "ResourceID",
    "ResourceProduct",
    "ResourceScope",
    "ResourceType",
]


class AuditListParams(TypedDict, total=False):
    before: Required[Annotated[Union[str, date], PropertyInfo(format="iso8601")]]
    """Limits the returned results to logs older than the specified date.

    This can be a date string 2019-04-30 (interpreted in UTC) or an absolute
    timestamp that conforms to RFC3339.
    """

    since: Required[Annotated[Union[str, date], PropertyInfo(format="iso8601")]]
    """Limits the returned results to logs newer than the specified date.

    This can be a date string 2019-04-30 (interpreted in UTC) or an absolute
    timestamp that conforms to RFC3339.
    """

    id: ID

    action_result: ActionResult

    action_type: ActionType

    actor_context: ActorContext

    actor_email: ActorEmail

    actor_id: ActorID

    actor_ip_address: ActorIPAddress

    actor_token_id: ActorTokenID

    actor_token_name: ActorTokenName

    actor_type: ActorType

    cursor: str
    """The cursor is an opaque token used to paginate through large sets of records.

    It indicates the position from which to continue when requesting the next set of
    records. A valid cursor value can be obtained from the cursor object in the
    result_info structure of a previous response.
    """

    direction: Literal["desc", "asc"]
    """Sets sorting order."""

    limit: float
    """The number limits the objects to return.

    The cursor attribute may be used to iterate over the next batch of objects if
    there are more than the limit.
    """

    raw_cf_rayid: Annotated[RawCfRayID, PropertyInfo(alias="raw_cf_ray_id")]

    raw_method: RawMethod

    raw_status_code: RawStatusCode

    raw_uri: RawURI

    resource_id: ResourceID

    resource_product: ResourceProduct

    resource_scope: ResourceScope

    resource_type: ResourceType


_IDReservedKeywords = TypedDict(
    "_IDReservedKeywords",
    {
        "not": SequenceNotStr[str],
    },
    total=False,
)


class ID(_IDReservedKeywords, total=False):
    pass


_ActionResultReservedKeywords = TypedDict(
    "_ActionResultReservedKeywords",
    {
        "not": List[Literal["success", "failure"]],
    },
    total=False,
)


class ActionResult(_ActionResultReservedKeywords, total=False):
    pass


_ActionTypeReservedKeywords = TypedDict(
    "_ActionTypeReservedKeywords",
    {
        "not": List[Literal["create", "delete", "view", "update"]],
    },
    total=False,
)


class ActionType(_ActionTypeReservedKeywords, total=False):
    pass


_ActorContextReservedKeywords = TypedDict(
    "_ActorContextReservedKeywords",
    {
        "not": List[Literal["api_key", "api_token", "dash", "oauth", "origin_ca_key"]],
    },
    total=False,
)


class ActorContext(_ActorContextReservedKeywords, total=False):
    pass


_ActorEmailReservedKeywords = TypedDict(
    "_ActorEmailReservedKeywords",
    {
        "not": SequenceNotStr[str],
    },
    total=False,
)


class ActorEmail(_ActorEmailReservedKeywords, total=False):
    pass


_ActorIDReservedKeywords = TypedDict(
    "_ActorIDReservedKeywords",
    {
        "not": SequenceNotStr[str],
    },
    total=False,
)


class ActorID(_ActorIDReservedKeywords, total=False):
    pass


_ActorIPAddressReservedKeywords = TypedDict(
    "_ActorIPAddressReservedKeywords",
    {
        "not": SequenceNotStr[str],
    },
    total=False,
)


class ActorIPAddress(_ActorIPAddressReservedKeywords, total=False):
    pass


_ActorTokenIDReservedKeywords = TypedDict(
    "_ActorTokenIDReservedKeywords",
    {
        "not": SequenceNotStr[str],
    },
    total=False,
)


class ActorTokenID(_ActorTokenIDReservedKeywords, total=False):
    pass


_ActorTokenNameReservedKeywords = TypedDict(
    "_ActorTokenNameReservedKeywords",
    {
        "not": SequenceNotStr[str],
    },
    total=False,
)


class ActorTokenName(_ActorTokenNameReservedKeywords, total=False):
    pass


_ActorTypeReservedKeywords = TypedDict(
    "_ActorTypeReservedKeywords",
    {
        "not": List[Literal["cloudflare_admin", "system", "user"]],
    },
    total=False,
)


class ActorType(_ActorTypeReservedKeywords, total=False):
    pass


_RawCfRayIDReservedKeywords = TypedDict(
    "_RawCfRayIDReservedKeywords",
    {
        "not": SequenceNotStr[str],
    },
    total=False,
)


class RawCfRayID(_RawCfRayIDReservedKeywords, total=False):
    pass


_RawMethodReservedKeywords = TypedDict(
    "_RawMethodReservedKeywords",
    {
        "not": SequenceNotStr[str],
    },
    total=False,
)


class RawMethod(_RawMethodReservedKeywords, total=False):
    pass


_RawStatusCodeReservedKeywords = TypedDict(
    "_RawStatusCodeReservedKeywords",
    {
        "not": Iterable[int],
    },
    total=False,
)


class RawStatusCode(_RawStatusCodeReservedKeywords, total=False):
    pass


_RawURIReservedKeywords = TypedDict(
    "_RawURIReservedKeywords",
    {
        "not": SequenceNotStr[str],
    },
    total=False,
)


class RawURI(_RawURIReservedKeywords, total=False):
    pass


_ResourceIDReservedKeywords = TypedDict(
    "_ResourceIDReservedKeywords",
    {
        "not": SequenceNotStr[str],
    },
    total=False,
)


class ResourceID(_ResourceIDReservedKeywords, total=False):
    pass


_ResourceProductReservedKeywords = TypedDict(
    "_ResourceProductReservedKeywords",
    {
        "not": SequenceNotStr[str],
    },
    total=False,
)


class ResourceProduct(_ResourceProductReservedKeywords, total=False):
    pass


_ResourceScopeReservedKeywords = TypedDict(
    "_ResourceScopeReservedKeywords",
    {
        "not": List[Literal["organizations"]],
    },
    total=False,
)


class ResourceScope(_ResourceScopeReservedKeywords, total=False):
    pass


_ResourceTypeReservedKeywords = TypedDict(
    "_ResourceTypeReservedKeywords",
    {
        "not": SequenceNotStr[str],
    },
    total=False,
)


class ResourceType(_ResourceTypeReservedKeywords, total=False):
    pass



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .traces import (
    TracesResource,
    AsyncTracesResource,
    TracesResourceWithRawResponse,
    AsyncTracesResourceWithRawResponse,
    TracesResourceWithStreamingResponse,
    AsyncTracesResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["RequestTracersResource", "AsyncRequestTracersResource"]


class RequestTracersResource(SyncAPIResource):
    @cached_property
    def traces(self) -> TracesResource:
        return TracesResource(self._client)

    @cached_property
    def with_raw_response(self) -> RequestTracersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RequestTracersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RequestTracersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RequestTracersResourceWithStreamingResponse(self)


class AsyncRequestTracersResource(AsyncAPIResource):
    @cached_property
    def traces(self) -> AsyncTracesResource:
        return AsyncTracesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncRequestTracersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRequestTracersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRequestTracersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRequestTracersResourceWithStreamingResponse(self)


class RequestTracersResourceWithRawResponse:
    def __init__(self, request_tracers: RequestTracersResource) -> None:
        self._request_tracers = request_tracers

    @cached_property
    def traces(self) -> TracesResourceWithRawResponse:
        return TracesResourceWithRawResponse(self._request_tracers.traces)


class AsyncRequestTracersResourceWithRawResponse:
    def __init__(self, request_tracers: AsyncRequestTracersResource) -> None:
        self._request_tracers = request_tracers

    @cached_property
    def traces(self) -> AsyncTracesResourceWithRawResponse:
        return AsyncTracesResourceWithRawResponse(self._request_tracers.traces)


class RequestTracersResourceWithStreamingResponse:
    def __init__(self, request_tracers: RequestTracersResource) -> None:
        self._request_tracers = request_tracers

    @cached_property
    def traces(self) -> TracesResourceWithStreamingResponse:
        return TracesResourceWithStreamingResponse(self._request_tracers.traces)


class AsyncRequestTracersResourceWithStreamingResponse:
    def __init__(self, request_tracers: AsyncRequestTracersResource) -> None:
        self._request_tracers = request_tracers

    @cached_property
    def traces(self) -> AsyncTracesResourceWithStreamingResponse:
        return AsyncTracesResourceWithStreamingResponse(self._request_tracers.traces)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from pydantic import Field as FieldInfo

from ...._utils import PropertyInfo
from ...._models import BaseModel

__all__ = [
    "VersionGetResponse",
    "Resources",
    "ResourcesBinding",
    "ResourcesBindingWorkersBindingKindAI",
    "ResourcesBindingWorkersBindingKindAISearch",
    "ResourcesBindingWorkersBindingKindAISearchNamespace",
    "ResourcesBindingWorkersBindingKindAnalyticsEngine",
    "ResourcesBindingWorkersBindingKindAssets",
    "ResourcesBindingWorkersBindingKindBrowser",
    "ResourcesBindingWorkersBindingKindD1",
    "ResourcesBindingWorkersBindingKindDataBlob",
    "ResourcesBindingWorkersBindingKindDispatchNamespace",
    "ResourcesBindingWorkersBindingKindDispatchNamespaceOutbound",
    "ResourcesBindingWorkersBindingKindDispatchNamespaceOutboundParam",
    "ResourcesBindingWorkersBindingKindDispatchNamespaceOutboundWorker",
    "ResourcesBindingWorkersBindingKindDurableObjectNamespace",
    "ResourcesBindingWorkersBindingKindHyperdrive",
    "ResourcesBindingWorkersBindingKindInherit",
    "ResourcesBindingWorkersBindingKindImages",
    "ResourcesBindingWorkersBindingKindJson",
    "ResourcesBindingWorkersBindingKindKVNamespace",
    "ResourcesBindingWorkersBindingKindMedia",
    "ResourcesBindingWorkersBindingKindMTLSCertificate",
    "ResourcesBindingWorkersBindingKindPlainText",
    "ResourcesBindingWorkersBindingKindPipelines",
    "ResourcesBindingWorkersBindingKindQueue",
    "ResourcesBindingWorkersBindingKindRatelimit",
    "ResourcesBindingWorkersBindingKindRatelimitSimple",
    "ResourcesBindingWorkersBindingKindR2Bucket",
    "ResourcesBindingWorkersBindingKindSecretText",
    "ResourcesBindingWorkersBindingKindSendEmail",
    "ResourcesBindingWorkersBindingKindService",
    "ResourcesBindingWorkersBindingKindTextBlob",
    "ResourcesBindingWorkersBindingKindVectorize",
    "ResourcesBindingWorkersBindingKindVersionMetadata",
    "ResourcesBindingWorkersBindingKindSecretsStoreSecret",
    "ResourcesBindingWorkersBindingKindFlagship",
    "ResourcesBindingWorkersBindingKindSecretKey",
    "ResourcesBindingWorkersBindingKindWorkflow",
    "ResourcesBindingWorkersBindingKindWasmModule",
    "ResourcesBindingWorkersBindingKindVPCService",
    "ResourcesBindingWorkersBindingKindVPCNetwork",
    "ResourcesScript",
    "ResourcesScriptNamedHandler",
    "ResourcesScriptRuntime",
    "ResourcesScriptRuntimeLimits",
    "Metadata",
]


class ResourcesBindingWorkersBindingKindAI(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["ai"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindAISearch(BaseModel):
    instance_name: str
    """The user-chosen instance name.

    Must exist at deploy time. The worker can search, chat, update, and manage
    items/jobs on this instance.
    """

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["ai_search"]
    """The kind of resource that the binding provides."""

    namespace: Optional[str] = None
    """The namespace the instance belongs to.

    Defaults to "default" if omitted. Customers who don't use namespaces can simply
    omit this field.
    """


class ResourcesBindingWorkersBindingKindAISearchNamespace(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    namespace: str
    """The user-chosen namespace name.

    Must exist before deploy -- Wrangler handles auto-creation on deploy failure (R2
    bucket pattern). The "default" namespace is auto-created by config-api for new
    accounts. Grants full access (CRUD + search + chat) to all instances within the
    namespace.
    """

    type: Literal["ai_search_namespace"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindAnalyticsEngine(BaseModel):
    dataset: str
    """The name of the dataset to bind to."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["analytics_engine"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindAssets(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["assets"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindBrowser(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["browser"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindD1(BaseModel):
    database_id: str
    """Identifier of the D1 database to bind to."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["d1"]
    """The kind of resource that the binding provides."""

    id: Optional[str] = None
    """Identifier of the D1 database to bind to."""


class ResourcesBindingWorkersBindingKindDataBlob(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    part: str
    """The name of the file containing the data content.

    Only accepted for `service worker syntax` Workers.
    """

    type: Literal["data_blob"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindDispatchNamespaceOutboundParam(BaseModel):
    name: str
    """Name of the parameter."""


class ResourcesBindingWorkersBindingKindDispatchNamespaceOutboundWorker(BaseModel):
    """Outbound worker."""

    entrypoint: Optional[str] = None
    """Entrypoint to invoke on the outbound worker."""

    environment: Optional[str] = None
    """Environment of the outbound worker."""

    service: Optional[str] = None
    """Name of the outbound worker."""


class ResourcesBindingWorkersBindingKindDispatchNamespaceOutbound(BaseModel):
    """Outbound worker."""

    params: Optional[List[ResourcesBindingWorkersBindingKindDispatchNamespaceOutboundParam]] = None
    """
    Pass information from the Dispatch Worker to the Outbound Worker through the
    parameters.
    """

    worker: Optional[ResourcesBindingWorkersBindingKindDispatchNamespaceOutboundWorker] = None
    """Outbound worker."""


class ResourcesBindingWorkersBindingKindDispatchNamespace(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    namespace: str
    """The name of the dispatch namespace."""

    type: Literal["dispatch_namespace"]
    """The kind of resource that the binding provides."""

    outbound: Optional[ResourcesBindingWorkersBindingKindDispatchNamespaceOutbound] = None
    """Outbound worker."""


class ResourcesBindingWorkersBindingKindDurableObjectNamespace(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["durable_object_namespace"]
    """The kind of resource that the binding provides."""

    class_name: Optional[str] = None
    """The exported class name of the Durable Object."""

    dispatch_namespace: Optional[str] = None
    """The dispatch namespace the Durable Object script belongs to."""

    environment: Optional[str] = None
    """The environment of the script_name to bind to."""

    namespace_id: Optional[str] = None
    """Namespace identifier tag."""

    script_name: Optional[str] = None
    """
    The script where the Durable Object is defined, if it is external to this
    Worker.
    """


class ResourcesBindingWorkersBindingKindHyperdrive(BaseModel):
    id: str
    """Identifier of the Hyperdrive connection to bind to."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["hyperdrive"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindInherit(BaseModel):
    name: str
    """The name of the inherited binding."""

    type: Literal["inherit"]
    """The kind of resource that the binding provides."""

    old_name: Optional[str] = None
    """The old name of the inherited binding.

    If set, the binding will be renamed from `old_name` to `name` in the new
    version. If not set, the binding will keep the same name between versions.
    """

    version_id: Optional[str] = None
    """
    Identifier for the version to inherit the binding from, which can be the version
    ID or the literal "latest" to inherit from the latest version. Defaults to
    inheriting the binding from the latest version.
    """


class ResourcesBindingWorkersBindingKindImages(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["images"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindJson(BaseModel):
    json_: object = FieldInfo(alias="json")
    """JSON data to use."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["json"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindKVNamespace(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    namespace_id: str
    """Namespace identifier tag."""

    type: Literal["kv_namespace"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindMedia(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["media"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindMTLSCertificate(BaseModel):
    certificate_id: str
    """Identifier of the certificate to bind to."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["mtls_certificate"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindPlainText(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    text: str
    """The text value to use."""

    type: Literal["plain_text"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindPipelines(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    pipeline: str
    """Name of the Pipeline to bind to."""

    type: Literal["pipelines"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindQueue(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    queue_name: str
    """Name of the Queue to bind to."""

    type: Literal["queue"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindRatelimitSimple(BaseModel):
    """The rate limit configuration."""

    limit: float
    """The limit (requests per period)."""

    period: int
    """The period in seconds."""

    mitigation_timeout: Optional[int] = None
    """
    Duration in seconds to apply the mitigation action after the rate limit is
    exceeded. Valid values are 0 (disabled), 10, or multiples of 60 up to 86400.
    Must be greater than or equal to the period when non-zero.
    """


class ResourcesBindingWorkersBindingKindRatelimit(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    namespace_id: str
    """Identifier of the rate limit namespace to bind to."""

    simple: ResourcesBindingWorkersBindingKindRatelimitSimple
    """The rate limit configuration."""

    type: Literal["ratelimit"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindR2Bucket(BaseModel):
    bucket_name: str
    """R2 bucket to bind to."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["r2_bucket"]
    """The kind of resource that the binding provides."""

    jurisdiction: Optional[Literal["eu", "fedramp", "fedramp-high"]] = None
    """
    The
    [jurisdiction](https://developers.cloudflare.com/r2/reference/data-location/#jurisdictional-restrictions)
    of the R2 bucket.
    """


class ResourcesBindingWorkersBindingKindSecretText(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["secret_text"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindSendEmail(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["send_email"]
    """The kind of resource that the binding provides."""

    allowed_destination_addresses: Optional[List[str]] = None
    """List of allowed destination addresses."""

    allowed_sender_addresses: Optional[List[str]] = None
    """List of allowed sender addresses."""

    destination_address: Optional[str] = None
    """Destination address for the email."""


class ResourcesBindingWorkersBindingKindService(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    service: str
    """Name of Worker to bind to."""

    type: Literal["service"]
    """The kind of resource that the binding provides."""

    entrypoint: Optional[str] = None
    """Entrypoint to invoke on the target Worker."""

    environment: Optional[str] = None
    """Optional environment if the Worker utilizes one."""


class ResourcesBindingWorkersBindingKindTextBlob(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    part: str
    """The name of the file containing the text content.

    Only accepted for `service worker syntax` Workers.
    """

    type: Literal["text_blob"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindVectorize(BaseModel):
    index_name: str
    """Name of the Vectorize index to bind to."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["vectorize"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindVersionMetadata(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["version_metadata"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindSecretsStoreSecret(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    secret_name: str
    """Name of the secret in the store."""

    store_id: str
    """ID of the store containing the secret."""

    type: Literal["secrets_store_secret"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindFlagship(BaseModel):
    app_id: str
    """ID of the Flagship app to bind to for feature flag evaluation."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["flagship"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindSecretKey(BaseModel):
    algorithm: object
    """Algorithm-specific key parameters.

    [Learn more](https://developer.mozilla.org/en-US/docs/Web/API/SubtleCrypto/importKey#algorithm).
    """

    format: Literal["raw", "pkcs8", "spki", "jwk"]
    """Data format of the key.

    [Learn more](https://developer.mozilla.org/en-US/docs/Web/API/SubtleCrypto/importKey#format).
    """

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["secret_key"]
    """The kind of resource that the binding provides."""

    usages: List[Literal["encrypt", "decrypt", "sign", "verify", "deriveKey", "deriveBits", "wrapKey", "unwrapKey"]]
    """Allowed operations with the key.

    [Learn more](https://developer.mozilla.org/en-US/docs/Web/API/SubtleCrypto/importKey#keyUsages).
    """


class ResourcesBindingWorkersBindingKindWorkflow(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["workflow"]
    """The kind of resource that the binding provides."""

    workflow_name: str
    """Name of the Workflow to bind to."""

    class_name: Optional[str] = None
    """Class name of the Workflow.

    Should only be provided if the Workflow belongs to this script.
    """

    script_name: Optional[str] = None
    """Script name that contains the Workflow.

    If not provided, defaults to this script name.
    """


class ResourcesBindingWorkersBindingKindWasmModule(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    part: str
    """The name of the file containing the WebAssembly module content.

    Only accepted for `service worker syntax` Workers.
    """

    type: Literal["wasm_module"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindVPCService(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    service_id: str
    """Identifier of the VPC service to bind to."""

    type: Literal["vpc_service"]
    """The kind of resource that the binding provides."""


class ResourcesBindingWorkersBindingKindVPCNetwork(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["vpc_network"]
    """The kind of resource that the binding provides."""

    network_id: Optional[str] = None
    """Identifier of the network to bind to.

    Only "cf1:network" is currently supported. Mutually exclusive with tunnel_id.
    """

    tunnel_id: Optional[str] = None
    """UUID of the Cloudflare Tunnel to bind to. Mutually exclusive with network_id."""


ResourcesBinding: TypeAlias = Annotated[
    Union[
        ResourcesBindingWorkersBindingKindAI,
        ResourcesBindingWorkersBindingKindAISearch,
        ResourcesBindingWorkersBindingKindAISearchNamespace,
        ResourcesBindingWorkersBindingKindAnalyticsEngine,
        ResourcesBindingWorkersBindingKindAssets,
        ResourcesBindingWorkersBindingKindBrowser,
        ResourcesBindingWorkersBindingKindD1,
        ResourcesBindingWorkersBindingKindDataBlob,
        ResourcesBindingWorkersBindingKindDispatchNamespace,
        ResourcesBindingWorkersBindingKindDurableObjectNamespace,
        ResourcesBindingWorkersBindingKindHyperdrive,
        ResourcesBindingWorkersBindingKindInherit,
        ResourcesBindingWorkersBindingKindImages,
        ResourcesBindingWorkersBindingKindJson,
        ResourcesBindingWorkersBindingKindKVNamespace,
        ResourcesBindingWorkersBindingKindMedia,
        ResourcesBindingWorkersBindingKindMTLSCertificate,
        ResourcesBindingWorkersBindingKindPlainText,
        ResourcesBindingWorkersBindingKindPipelines,
        ResourcesBindingWorkersBindingKindQueue,
        ResourcesBindingWorkersBindingKindRatelimit,
        ResourcesBindingWorkersBindingKindR2Bucket,
        ResourcesBindingWorkersBindingKindSecretText,
        ResourcesBindingWorkersBindingKindSendEmail,
        ResourcesBindingWorkersBindingKindService,
        ResourcesBindingWorkersBindingKindTextBlob,
        ResourcesBindingWorkersBindingKindVectorize,
        ResourcesBindingWorkersBindingKindVersionMetadata,
        ResourcesBindingWorkersBindingKindSecretsStoreSecret,
        ResourcesBindingWorkersBindingKindFlagship,
        ResourcesBindingWorkersBindingKindSecretKey,
        ResourcesBindingWorkersBindingKindWorkflow,
        ResourcesBindingWorkersBindingKindWasmModule,
        ResourcesBindingWorkersBindingKindVPCService,
        ResourcesBindingWorkersBindingKindVPCNetwork,
    ],
    PropertyInfo(discriminator="type"),
]


class ResourcesScriptNamedHandler(BaseModel):
    handlers: Optional[List[str]] = None
    """The names of handlers exported as part of the named export."""

    name: Optional[str] = None
    """The name of the exported class or entrypoint."""


class ResourcesScript(BaseModel):
    etag: Optional[str] = None
    """Hashed script content"""

    handlers: Optional[List[str]] = None
    """The names of handlers exported as part of the default export."""

    last_deployed_from: Optional[str] = None
    """The client most recently used to deploy this Worker."""

    named_handlers: Optional[List[ResourcesScriptNamedHandler]] = None
    """
    Named exports, such as Durable Object class implementations and named
    entrypoints.
    """


class ResourcesScriptRuntimeLimits(BaseModel):
    """Resource limits for the Worker."""

    cpu_ms: Optional[int] = None
    """The amount of CPU time this Worker can use in milliseconds."""


class ResourcesScriptRuntime(BaseModel):
    """Runtime configuration for the Worker."""

    compatibility_date: Optional[str] = None
    """Date indicating targeted support in the Workers runtime.

    Backwards incompatible fixes to the runtime following this date will not affect
    this Worker.
    """

    compatibility_flags: Optional[List[str]] = None
    """Flags that enable or disable certain features in the Workers runtime."""

    limits: Optional[ResourcesScriptRuntimeLimits] = None
    """Resource limits for the Worker."""

    migration_tag: Optional[str] = None
    """
    The tag of the Durable Object migration that was most recently applied for this
    Worker.
    """

    usage_model: Optional[Literal["bundled", "unbound", "standard"]] = None
    """Usage model for the Worker invocations."""


class Resources(BaseModel):
    bindings: Optional[List[ResourcesBinding]] = None
    """List of bindings attached to a Worker.

    You can find more about bindings on our docs:
    https://developers.cloudflare.com/workers/configuration/multipart-upload-metadata/#bindings.
    """

    script: Optional[ResourcesScript] = None

    script_runtime: Optional[ResourcesScriptRuntime] = None
    """Runtime configuration for the Worker."""


class Metadata(BaseModel):
    author_email: Optional[str] = None
    """Email of the user who created the version."""

    author_id: Optional[str] = None
    """Identifier of the user who created the version."""

    created_on: Optional[str] = None
    """When the version was created."""

    has_preview: Optional[bool] = FieldInfo(alias="hasPreview", default=None)
    """Whether the version can be previewed."""

    modified_on: Optional[str] = None
    """When the version was last modified."""

    source: Optional[
        Literal[
            "unknown",
            "api",
            "wrangler",
            "terraform",
            "dash",
            "dash_template",
            "integration",
            "quick_editor",
            "playground",
            "workersci",
        ]
    ] = None
    """The source of the version upload."""


class VersionGetResponse(BaseModel):
    resources: Resources

    id: Optional[str] = None
    """Unique identifier for the version."""

    metadata: Optional[Metadata] = None

    number: Optional[float] = None
    """Sequential version number."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["HostnameAssociationGetParams"]


class HostnameAssociationGetParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    mtls_certificate_id: str
    """
    The UUID to match against for a certificate that was uploaded to the mTLS
    Certificate Management endpoint. If no mtls_certificate_id is given, the results
    will be the hostnames associated to your active Cloudflare Managed CA.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal, Required, TypedDict

from ..approval_group_param import ApprovalGroupParam

__all__ = ["PolicyUpdateParams", "ConnectionRules", "ConnectionRulesRDP", "MfaConfig"]


class PolicyUpdateParams(TypedDict, total=False):
    app_id: Required[str]
    """UUID."""

    account_id: str
    """The Account ID to use for this endpoint. Mutually exclusive with the Zone ID."""

    zone_id: str
    """The Zone ID to use for this endpoint. Mutually exclusive with the Account ID."""

    approval_groups: Iterable[ApprovalGroupParam]
    """Administrators who can approve a temporary authentication request."""

    approval_required: bool
    """
    Requires the user to request access from an administrator at the start of each
    session.
    """

    connection_rules: ConnectionRules
    """
    The rules that define how users may connect to targets secured by your
    application.
    """

    isolation_required: bool
    """
    Require this application to be served in an isolated browser for users matching
    this policy. 'Client Web Isolation' must be on for the account in order to use
    this feature.
    """

    mfa_config: MfaConfig
    """Configures multi-factor authentication (MFA) settings."""

    precedence: int
    """The order of execution for this policy.

    Must be unique for each policy within an app.
    """

    purpose_justification_prompt: str
    """A custom message that will appear on the purpose justification screen."""

    purpose_justification_required: bool
    """Require users to enter a justification when they log in to the application."""

    session_duration: str
    """The amount of time that tokens issued for the application will be valid.

    Must be in the format `300ms` or `2h45m`. Valid time units are: ns, us (or µs),
    ms, s, m, h.
    """


class ConnectionRulesRDP(TypedDict, total=False):
    """The RDP-specific rules that define clipboard behavior for RDP connections."""

    allowed_clipboard_local_to_remote_formats: List[Literal["text"]]
    """
    Clipboard formats allowed when copying from local machine to remote RDP session.
    """

    allowed_clipboard_remote_to_local_formats: List[Literal["text"]]
    """
    Clipboard formats allowed when copying from remote RDP session to local machine.
    """


class ConnectionRules(TypedDict, total=False):
    """
    The rules that define how users may connect to targets secured by your application.
    """

    rdp: ConnectionRulesRDP
    """The RDP-specific rules that define clipboard behavior for RDP connections."""


class MfaConfig(TypedDict, total=False):
    """Configures multi-factor authentication (MFA) settings."""

    allowed_authenticators: List[Literal["totp", "biometrics", "security_key"]]
    """Lists the MFA methods that users can authenticate with."""

    mfa_disabled: bool
    """Indicates whether to disable MFA for this resource.

    This option is available at the application and policy level.
    """

    session_duration: str
    """Defines the duration of an MFA session.

    Must be in minutes (m) or hours (h). Minimum: 0m. Maximum: 720h (30 days).
    Examples:`5m` or `24h`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["CertificateCreateParams"]


class CertificateCreateParams(TypedDict, total=False):
    account_id: Required[str]

    validity_period_days: int
    """Sets the certificate validity period in days (range: 1-10,950 days / ~30 years).

    Defaults to 1,825 days (5 years). **Important**: This field is only settable
    during the certificate creation. Certificates becomes immutable after creation -
    use the `/activate` and `/deactivate` endpoints to manage certificate lifecycle.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .targets import (
    TargetsResource,
    AsyncTargetsResource,
    TargetsResourceWithRawResponse,
    AsyncTargetsResourceWithRawResponse,
    TargetsResourceWithStreamingResponse,
    AsyncTargetsResourceWithStreamingResponse,
)
from .infrastructure import (
    InfrastructureResource,
    AsyncInfrastructureResource,
    InfrastructureResourceWithRawResponse,
    AsyncInfrastructureResourceWithRawResponse,
    InfrastructureResourceWithStreamingResponse,
    AsyncInfrastructureResourceWithStreamingResponse,
)

__all__ = [
    "TargetsResource",
    "AsyncTargetsResource",
    "TargetsResourceWithRawResponse",
    "AsyncTargetsResourceWithRawResponse",
    "TargetsResourceWithStreamingResponse",
    "AsyncTargetsResourceWithStreamingResponse",
    "InfrastructureResource",
    "AsyncInfrastructureResource",
    "InfrastructureResourceWithRawResponse",
    "AsyncInfrastructureResourceWithRawResponse",
    "InfrastructureResourceWithStreamingResponse",
    "AsyncInfrastructureResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["SubmissionListParams"]


class SubmissionListParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    end: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """The end of the search date range. Defaults to `now`."""

    escalated_from_user: bool
    """When true, return only submissions that were escalated by an end user (vs.

    by the security team). When false, return only submissions that were not
    escalated by an end user. When omitted, no filter is applied.
    """

    original_disposition: Literal["MALICIOUS", "SUSPICIOUS", "SPOOF", "SPAM", "BULK", "NONE"]

    outcome_disposition: Literal["MALICIOUS", "SUSPICIOUS", "SPOOF", "SPAM", "BULK", "NONE"]

    page: int
    """Current page within paginated list of results."""

    per_page: int
    """The number of results per page. Maximum value is 1000."""

    query: Optional[str]

    requested_disposition: Literal["MALICIOUS", "SUSPICIOUS", "SPOOF", "SPAM", "BULK", "NONE"]

    start: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """The beginning of the search date range. Defaults to `now - 30 days`."""

    status: str

    submission_id: str

    type: Literal["TEAM", "USER"]
